<template>
  <win-md class="stock-edit" :title="`${setting.title}出库`" @close="close" :width="'800px'">
    <el-form class="form" ref="ruleForm" :model="formData" :rules="rules" label-width="120px">
      <div class="main-w">
        <el-row :gutter="24" class="headerHeight">
          <el-col :span="12">
            <el-form-item label="出库仓库" prop="warehouseId">
              <el-select v-model="formData.warehouseId" placeholder="请选择" style="width: 100%">
                <el-option
                  v-for="item in warehouseOptions"
                  :key="item.id"
                  :label="item.warehouseName"
                  :value="item.id"
                />
              </el-select>
            </el-form-item>
          </el-col>
          <el-col :span="12">
            <el-form-item label="出库时间" prop="dealTime">
              <el-date-picker
                v-model="formData.dealTime"
                type="datetime"
                value="yyyy-MM-dd HH:mm:ss"
                value-format="yyyyMMddHHmmss"
                placeholder="请选择日期"
                style="width: 100%"
              >
              </el-date-picker>
            </el-form-item>
          </el-col>
        </el-row>
        <el-row :gutter="24" class="headerHeight">
          <el-col :span="24">
            <el-form-item label="出库手续" prop="procureDoc">
              <upload ref="uploadRef" :settings="uploadSettings" @on-change="uploadChange"></upload>
            </el-form-item>
          </el-col>
        </el-row>
        <div class="goods-card" v-for="(goodsItem, goodsIndex) in formData.goods" :key="goodsIndex">
          <el-row :gutter="24">
            <el-col :span="12">
              <el-form-item
                label="物品分类"
                :prop="`goods[${goodsIndex}].baseCategoryIds`"
                :rules="{
                  required: true,
                  message: '请选择',
                  trigger: 'change',
                }"
              >
                <el-cascader
                  v-model="goodsItem.baseCategoryIds"
                  :options="categoryOptions"
                  :props="{ value: 'id' }"
                  @change="categoryChange($event, goodsIndex)"
                  style="width: 100%"
                ></el-cascader>
              </el-form-item>
            </el-col>
            <el-col :span="12">
              <el-form-item
                label="物品名称"
                :prop="`goods[${goodsIndex}].baseGoodsTemplateId`"
                :rules="rules.baseGoodsTemplateId"
              >
                <el-select
                  v-model="goodsItem.baseGoodsTemplateId"
                  placeholder="请先择物品分类"
                  filterable
                  :disabled="!goodsItem.baseCategoryId"
                  style="width: 100%"
                  @change="goodsTemplateChange($event, goodsIndex)"
                >
                  <el-option
                    v-for="item in goodsItem.goodsOptions"
                    :key="item.id"
                    :label="item.goodsName"
                    :value="item.id"
                  />
                </el-select>
              </el-form-item>
            </el-col>
          </el-row>
          <el-row :gutter="24">
            <el-col :span="12">
              <el-form-item
                label="规格型号"
                :prop="`goods[${goodsIndex}].modelsIds`"
                :rules="{
                  required: true,
                  message: '请选择',
                  trigger: 'change',
                }"
              >
                <el-select
                  v-model="goodsItem.modelsIds"
                  multiple
                  placeholder="请先择物品名称"
                  :disabled="!goodsItem.baseCategoryId"
                  @change="modelChange($event, goodsIndex)"
                  @remove-tag="modelRemoveTag($event, goodsIndex)"
                  style="width: 100%"
                >
                  <el-option
                    v-for="item in goodsItem.modelsOptions"
                    :key="item.id"
                    :label="item.modelName"
                    :value="item.id"
                  />
                </el-select>
              </el-form-item>
            </el-col>
          </el-row>
          <el-table :data="goodsItem.models" :stripe="true">
            <el-table-column prop="baseGoodsModelsId" label="规格型号" align="center">
              <template slot-scope="scope">
                {{ getGoodsModelsName(scope.row.baseGoodsModelsId) }}
              </template>
            </el-table-column>
            <el-table-column label="单位" align="center">
              <template slot-scope="scope">
                {{ scope.row.unit }}
              </template>
            </el-table-column>
            <el-table-column prop="price" label="现有库存" align="center">
              <template slot-scope="scope">
                {{ scope.row.worehouseCount }}
              </template>
            </el-table-column>
            <el-table-column prop="counts" label="出库数量" align="center">
              <template slot-scope="scope">
                <el-form-item
                  label-width="0"
                  style="margin-bottom: 0"
                  :prop="`goods[${goodsIndex}].models[${scope.$index}].counts`"
                  :rules="rules.counts"
                >
                  <el-input
                    type="number"
                    v-model="scope.row.counts"
                    @change="countsChange($event, goodsIndex, scope.$index)"
                  ></el-input>
                </el-form-item>
              </template>
            </el-table-column>
          </el-table>
          <div class="btn-group">
            <el-button
              v-if="formData.goods.length > 1"
              name="移除"
              type="danger"
              plain
              size="mini"
              @click="removeGoods(goodsIndex)"
              >移除</el-button
            >
            <el-button
              v-if="formData.goods.length - 1 == goodsIndex"
              name="新增物品"
              type="primary"
              plain
              size="mini"
              @click="addGoods"
              >新增物品</el-button
            >
          </div>
        </div>
      </div>
    </el-form>
    <div slot="footer" align="center" class="dialog-footer">
      <my-button name="取消" site="form" @click="close" />
      <el-button name="保存" type="primary" size="medium" :loading="btnloading" @click="handleSubmit">保存</el-button>
    </div>
  </win-md>
</template>
<script>
import {
  goodsTemplate,
  goodsModel,
  selectTenantWarehouse,
  warehouseSelectNumber,
} from '@/api/baseSetting/finsystenant';
import { outputAdd } from '@/api/stock/accessStock';
import { getTree } from '@/api/foudation/classification';
import MyButton from '@/components/myButton/myButton';
import winMd from '@/components/win/win-md';
import upload from '@/components/upload/index';
import { getUploadUrl } from '@/utils/base';
import { mapGetters } from 'vuex';

export default {
  components: { MyButton, winMd, upload },
  props: {
    setting: {
      type: Object,
      default: () => {},
    },
  },
  data() {
    // 自定义校验
    // 物品重复校验
    var checkGoodsTemplateId = (rule, value, callback) => {
      if (!value) {
        return callback(new Error('请选择'));
      }
      let temp = this.formData.goods.filter((v) => v.baseGoodsTemplateId == value);
      if (!temp || temp.length != 1) {
        return callback(new Error('已选过此物品'));
      }
      callback();
    };
    return {
      loading: false,
      btnloading: false,
      warehouseOptions: [], // 出库仓库列表
      categoryOptions: [], // 物品分类列表
      modelList: [], //型号列表
      formData: {
        warehouseId: '', // 出库仓库id
        dealTime: '', // 出库时间
        procureDoc: '',
        goods: [],
      },
      goodsItem: {
        baseCategoryIds: '', // 分类编号数组
        baseCategoryId: '', // 分类编号
        baseGoodsTemplateId: '', // 物品模版编号
        goodsTemplateName: '', // 物品模版名称
        goodsOptions: [], // 物品列表select
        modelsOptions: [], //规格型号select
        models: [], // 物品名称
        modelsIds: [], //规格型号
      },
      modelsItem: {
        baseGoodsModelsId: '', // 规格型号编号
        worehouseCount: 0,
        counts: null, // 操作数量
        unit: null, //单位
      },
      rules: {
        warehouseId: [{ required: true, message: '请选择', trigger: 'change' }],
        dealTime: [{ required: true, message: '请选择', trigger: 'change' }],
        buyType: [{ required: true, message: '请选择', trigger: 'change' }],

        baseCategoryIds: [{ required: true, message: '请选择', trigger: 'change' }],
        baseGoodsTemplateId: [{ validator: checkGoodsTemplateId, trigger: ['blur', 'change'] }],
        modelsIds: [{ required: true, message: '请选择', trigger: 'change' }],
        counts: [{ required: true, message: '请输入', trigger: 'change' }],
      },

      uploadSettings: {
        title: '上传',
        max: 20, // 最大大小，单位M
        num: 10, // 支持上传图片个数
        accept: '.jpg,.png', // 限制格式
        tip: '', // 提示 默认：`只能上传${this.defaultSettings.num}个${this.defaultSettings.accept}文件，且不超过${this.defaultSettings.max}kb`
        uploadUrl: getUploadUrl(), // 上传路径
        multiple: true, // 是否支持批量上传
        disabled: false, // 是否禁用
        type: 'picture', // text/picture
      },
    };
  },
  computed: {
    ...mapGetters(['userInfo']),
  },
  created() {
    this.init();
  },
  methods: {
    async init() {
      this.getWarehouseList();
      this.getgoodsTemplate();
      this.getgoodsModel();
      // 获取物品分类列表
      const treeRes = await getTree();
      this.categoryOptions = this.removeEmptyChildren(treeRes);
    },

    // 获取出库仓库列表
    getWarehouseList() {
      selectTenantWarehouse({ agencyId: this.userInfo.tenantId })
        .then((res) => {
          this.warehouseOptions = res;
          if (res.length) {
            this.formData.goods.push(JSON.parse(JSON.stringify(this.goodsItem)));
          }
          if (this.warehouseOptions.length && !this.formData.warehouseId) {
            // 默认选中第一个仓库
            this.formData.warehouseId = this.warehouseOptions[0].id;
          }
        })
        .catch((err) => {
          console.log('err', err);
        });
    },

    removeEmptyChildren(arr) {
      arr.forEach((item) => {
        if (!item.children || !item.children.length) {
          delete item.children;
        } else {
          this.removeEmptyChildren(item.children);
        }
      });
      return arr;
    },

    // 获取物品名称列表
    getgoodsTemplate(id, index) {
      goodsTemplate({ categoryId: id || '' }).then((res) => {
        if (index || index == 0) {
          this.$set(this.formData.goods[index], 'goodsOptions', res);
        } else {
          this.goodsTemplatelAll = res;
        }
      });
    },

    // 规格型号
    getgoodsModel(id, index) {
      goodsModel({ goodsTemplatesId: id || '' }).then((res) => {
        if (index || index == 0) {
          this.$set(this.formData.goods[index], 'modelsOptions', res);
        } else {
          this.goodsModelAll = res;
        }
      });
    },

    // 根据物品名称id获取名字
    getGoodsTemplateName(id) {
      let item = this.goodsTemplatelAll.find((v) => v.id == id);
      if (item) {
        return item.goodsName;
      }
      return;
    },

    // 根据规格型号id获取名字
    getGoodsModelsName(id) {
      let item = this.goodsModelAll.find((v) => v.id == id);
      if (item) {
        return item.modelName;
      }
      return;
    },

    // 物品分类选择
    categoryChange(e, index) {
      this.formData.goods[index].goodsOptions = [];
      this.formData.goods[index].baseGoodsTemplateId = '';
      this.formData.goods[index].goodsTemplateName = '';
      this.formData.goods[index].modelsOptions = [];
      this.formData.goods[index].modelsIds = [];
      this.formData.goods[index].models = [];

      this.formData.goods[index].baseCategoryId = e[e.length - 1];
      // 根据选中分类请求物品名称列表
      this.getgoodsTemplate(e[e.length - 1], index);
    },

    // 物品名称列表选择
    goodsTemplateChange(e, index) {
      this.formData.goods[index].modelsOptions = [];
      this.formData.goods[index].modelsIds = [];
      this.formData.goods[index].models = [];

      this.formData.goods[index].goodsTemplateName = this.getGoodsTemplateName(e);
      // 根据选中物品名称id获取规格型号列表
      this.getgoodsModel(e, index);
    },

    // 规格型号选择
    async modelChange(e, index) {
      let arr = [...this.formData.goods[index].models];
      let str = JSON.stringify(arr);
      for (let item of e) {
        if (str.indexOf(item) == -1) {
          let temp = this.goodsModelAll.find((v) => v.id == item);
          // 获取库存
          let num = await warehouseSelectNumber({
            warehouseId: this.formData.warehouseId,
            baseGoodsModelsId: item,
            warehouseType: 0,
            states: 1,
            buyType: null,
          });
          arr.push({ ...this.modelsItem, baseGoodsModelsId: item, unit: temp.unit, worehouseCount: num });
        }
      }
      this.formData.goods[index].models = arr;
    },

    // 规格型号移除
    modelRemoveTag(e, index) {
      let arr = this.formData.goods[index].models;
      let delIndex = arr.findIndex((v) => v.baseGoodsModelsId == e);
      this.formData.goods[index].models.splice(delIndex, 1);
    },

    // 上传
    uploadChange() {
      let arr = this.$refs.uploadRef.fileList;
      this.formData.procureDoc = JSON.stringify(arr);
    },

    // 点击新增物品
    addGoods() {
      this.formData.goods.push(JSON.parse(JSON.stringify(this.goodsItem)));
    },

    // 点击移除
    removeGoods(index) {
      this.formData.goods.splice(index, 1);
    },

    // 出库数量校验
    countsChange(e, goodsIndex, index) {
      const curItem = this.formData.goods[goodsIndex].models[index];
      const worehouseCount = curItem.worehouseCount;
      if (e == 0) {
        this.$message.warning('数额不能为0');
        curItem.counts = null;
      }
      if (e > worehouseCount) {
        this.$message.warning('数额超过现有库存');
        curItem.counts = worehouseCount > 0 ? worehouseCount : null;
      }
    },

    // 提交
    handleSubmit() {
      this.$refs['ruleForm'].validate((valid) => {
        if (valid) {
          this.btnloading = true;
          console.log('this.formData', this.formData);
          outputAdd(this.formData)
            .then((res) => {
              this.$message.success('保存成功！');
              this.btnloading = false;
              this.close();
              this.$emit('search');
            })
            .catch((err) => {
              console.log('edit err', err);
              this.btnloading = false;
              this.$message.error('保存失败');
            });
        } else {
          this.$message.error('校验未通过，请检查。');
          this.btnloading = false;
        }
      });
    },

    close() {
      this.formData = {
        warehouseId: '', // 出库仓库id
        dealTime: '', // 出库时间
        goods: [],
      };
      this.$emit('close');
    },
  },
};
</script>
<style scoped lang="scss">
@import url(../../index.scss);
</style>
